import 'dart:convert';
import 'package:flutter/material.dart';
import 'dart:async';
import 'dart:io';
import 'package:flutter/services.dart';
import 'package:only_for_trial/only_for_trial.dart';
import 'package:only_for_trial_example/ResultScreen.dart';
import 'package:only_for_trial_example/barcodeList.dart';
import 'package:only_for_trial_example/cardList.dart';

void main() {
  runApp(MaterialApp(
    home: MyApp(),
    debugShowCheckedModeBanner: false,
  ));
}

class MyApp extends StatefulWidget {
  const MyApp({super.key});
  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  dynamic _result = {"isValid",false};
  String mfile = "";
  List<File> images = [];
  Map<String,dynamic> countrySelected = {};
  String mrzCountryList = 'all';
  bool isMRZEnable = false;
  bool isBarcodeEnable = false;
  bool isBankCardEnable = false;
  List barcodeList = [];
  var count = 0;
  String orientation = 'portrait';
  List<Map<String, dynamic>> mrzList = [
    {"label": "Passport", "value": "passport_mrz"},
    {"label": "Mrz ID", "value": "id_mrz"},
    {"label": "Visa Card", "value": "visa_mrz"},
    {"label": "Other", "value": "other_mrz"}
  ];

  List<String> listOfCountryandMrz = [''];
  String mrzselected = '';
  dynamic sdkConfig;


  @override
  void initState() {
    super.initState();
    getMetaData();
  }

  Future<void> getMetaData() async{
    try {

      await OnlyForTrial.setFaceBlurPercentage(80);
      await OnlyForTrial.setHologramDetection(true);
      await OnlyForTrial.setLowLightTolerance(10);
      await OnlyForTrial.setMotionThreshold(25);
      await OnlyForTrial.setMinGlarePercentage(6);
      await OnlyForTrial.setMaxGlarePercentage(99);
      await OnlyForTrial.setBlurPercentage(60);
      await OnlyForTrial.setCameraFacing(0);
      await OnlyForTrial.isCheckPhotoCopy(false);

      await OnlyForTrial.SCAN_TITLE_OCR_FRONT("Scan Front side of ");
      await OnlyForTrial.SCAN_TITLE_OCR_BACK("Scan Back side of ");
      await OnlyForTrial.SCAN_TITLE_OCR("Scan ");
      await OnlyForTrial.SCAN_TITLE_MRZ_PDF417_FRONT("Scan Front Side of Document");
      await OnlyForTrial.SCAN_TITLE_MRZ_PDF417_BACK("Scan Back Side of Document");
      await OnlyForTrial.SCAN_TITLE_DLPLATE("Scan Number plate");
      await OnlyForTrial.SCAN_TITLE_BARCODE("Scan Barcode");
      await OnlyForTrial.SCAN_TITLE_BANKCARD("Scan BankCard");


      await OnlyForTrial.ACCURA_ERROR_CODE_MOTION("Keep Document Steady");
      await OnlyForTrial.ACCURA_ERROR_CODE_DOCUMENT_IN_FRAME("Keep document in frame");
      await OnlyForTrial.ACCURA_ERROR_CODE_BRING_DOCUMENT_IN_FRAME("Bring card near to frame");
      await OnlyForTrial.ACCURA_ERROR_CODE_PROCESSING("Processing");
      await OnlyForTrial.ACCURA_ERROR_CODE_BLUR_DOCUMENT("Blur detect in document");
      await OnlyForTrial.ACCURA_ERROR_CODE_FACE_BLUR("Blur detected over face");
      await OnlyForTrial.ACCURA_ERROR_CODE_GLARE_DOCUMENT("Glare detect in document");
      await OnlyForTrial.ACCURA_ERROR_CODE_HOLOGRAM("Hologram Detected");
      await OnlyForTrial.ACCURA_ERROR_CODE_DARK_DOCUMENT("Low lighting detected");
      await OnlyForTrial.ACCURA_ERROR_CODE_PHOTO_COPY_DOCUMENT("Can not accept Photo Copy Document");
      await OnlyForTrial.ACCURA_ERROR_CODE_FACE("Face not detected");
      await OnlyForTrial.ACCURA_ERROR_CODE_MRZ("MRZ not detected");
      await OnlyForTrial.ACCURA_ERROR_CODE_PASSPORT_MRZ("Passport MRZ not detected");
      await OnlyForTrial.ACCURA_ERROR_CODE_ID_MRZ("ID MRZ not detected");
      await OnlyForTrial.ACCURA_ERROR_CODE_VISA_MRZ("Visa MRZ not detected");
      await OnlyForTrial.ACCURA_ERROR_CODE_UPSIDE_DOWN_SIDE("Document is upside down. Place it properly");
      await OnlyForTrial.ACCURA_ERROR_CODE_WRONG_SIDE("Scanning wrong side of Document");
      await OnlyForTrial.isShowLogo(0);

      await OnlyForTrial.getMetaData().then((value) =>
          setupConfigData(json.decode(value)));
    }on PlatformException{}
    if (!mounted) return;
  }

  String getOrientation() {
    if(MediaQuery.of(context).orientation.toString().contains('landscape')){
      return 'landscape';
    }else{
      return 'portrait';
    }
  }

  Future<void> startBankCard() async{
    try{
      await OnlyForTrial.startBankCard().then((value) => {
        setState((){
          _result = json.decode(value);
          ResultScreen.result = _result;
        })
      });
    }on PlatformException{}
  }


  void setupConfigData(obj){
    setState((){
      sdkConfig = obj;
    });
    if(sdkConfig["isValid"] == true){
      List<String> tempList3=[];
      List barcodeData = [];
      print("setUpConfigData = ${obj}");

      List countryData = obj['countryList'];
      print("setUpConfigDatabvg = $countryData");
      print("objectsscscs ${sdkConfig['isMRZEnable']}");
      if(sdkConfig['isMRZEnable']) {
        for (var item in mrzList) {
          tempList3.add(item['label']);
        }
      }
      if(sdkConfig['isBankCardEnable']){
        tempList3.add('Bank Card');
      }
      if(sdkConfig['isBarcodeEnable']) {
        tempList3.add("Barcode");
        barcodeData = obj["barcodes"];
        print("mcidcme $barcodeData");
      }

      for (var item in countryData) {
        tempList3.add(item['name']);
      }

      setState((){
        barcodeList = barcodeData;
        listOfCountryandMrz = tempList3;
      });
    }
  }

  Future<void> startMRZ() async {
    try {
      var config = [
        {"enableLogs": false},
        mrzselected,
      ];
      print('startMRZ:- $config');
      await OnlyForTrial.startMRZ(config)
          .then((value) => {
        setState((){
          _result = json.decode(value);
          ResultScreen.Orientation = orientation;
          ResultScreen.result = _result;
          print("RESULT:- $_result");
        })
      }).onError((error, stackTrace) => {
        setState(() {
          print("njhvgchgb");
        })
      });
    } on PlatformException {}
  }


  @override
  Widget build(BuildContext context) {
    {
      print(MediaQuery.of(context).orientation.toString());
    }
    return MaterialApp(
        home: Scaffold(
            appBar: AppBar(
              leading: IconButton(
                icon: Icon(Icons.arrow_back, color: Colors.white),
                onPressed: () => Navigator.of(context).pop(),
              ),
              title: Text("Accura KYC"),
              actions: [TextButton(onPressed: (){
                count = count + 1;
                if(count%2 != 0){
                  SystemChrome.setPreferredOrientations([DeviceOrientation.landscapeLeft]);
                  orientation = "landscape";
                }
                else{
                  SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
                  orientation = "portrait";
                }
              }, child: count%2 == 0?Text("Landscape",style: TextStyle(color: Colors.white),)
                  :Text("Portrait",style: TextStyle(color: Colors.white),))],
              backgroundColor: Colors.red[800],
            ),
            resizeToAvoidBottomInset: true,
            body: SafeArea(child: Container(
              decoration: BoxDecoration(
                image: DecorationImage(
                    image: AssetImage("assets/images/background.png"),
                    fit: BoxFit.cover
                ),
              ),
              child:
              SingleChildScrollView(
                  child: Center(
                    child: Container(
                      child:
                      sdkConfig == null
                          ? Center(child: CircularProgressIndicator())
                          : !sdkConfig["isValid"]
                          ?
                      Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: [
                          Image.asset('assets/images/license.png',height: 150,width: 150,),
                          Text("License is not Valid",textAlign: TextAlign.center,)
                        ],
                      ) :
                      ListView.builder(
                          shrinkWrap: true,
                          physics: NeverScrollableScrollPhysics(),
                          scrollDirection: Axis.vertical,
                          itemCount: listOfCountryandMrz.length,
                          itemBuilder: (context,index){
                            return
                              SingleChildScrollView(
                                  child:
                                  Card(
                                      color: (listOfCountryandMrz[index] == 'Passport' || listOfCountryandMrz[index] == 'Visa Card'
                                          || listOfCountryandMrz[index] == 'Mrz ID' || listOfCountryandMrz[index] == 'Other'
                                          || listOfCountryandMrz[index] == 'Bank Card' || listOfCountryandMrz[index] == 'Barcode'
                                      )?Colors.grey:Colors.red[800],
                                      child:
                                      ListTile(
                                        title: Text(listOfCountryandMrz[index]
                                          ,style: TextStyle(fontSize: 20,fontWeight: FontWeight.bold,color: Colors.white),
                                        ),
                                        onTap: () async{
                                          if(listOfCountryandMrz[index] == 'Passport' || listOfCountryandMrz[index] == 'Visa Card'
                                              || listOfCountryandMrz[index] == 'Mrz ID' || listOfCountryandMrz[index] == 'Other'
                                              || listOfCountryandMrz[index] == 'Bank Card' || listOfCountryandMrz[index] == 'Barcode'
                                          ){
                                            if(listOfCountryandMrz[index] == 'Passport'){
                                              mrzselected = 'passport_mrz';
                                              print('passport');
                                              await startMRZ();
                                              Navigator.push(
                                                  context,
                                                  MaterialPageRoute(
                                                      builder: (context) => ResultScreen()));
                                            }
                                            else if(listOfCountryandMrz[index] == 'Visa Card'){
                                              mrzselected = 'visa_card';
                                              print('visa_card');
                                              await startMRZ();
                                              Navigator.push(
                                                  context,
                                                  MaterialPageRoute(
                                                      builder: (context) => ResultScreen()));
                                            }
                                            else if(listOfCountryandMrz[index] == 'Mrz ID'){
                                              mrzselected = 'id_mrz';
                                              print('ID');
                                              await startMRZ();
                                              Navigator.push(
                                                  context,
                                                  MaterialPageRoute(
                                                      builder: (context) => ResultScreen()));
                                            }else if(listOfCountryandMrz[index] == 'Bank Card'){
                                              await startBankCard();
                                              Navigator.push(
                                                  context,
                                                  MaterialPageRoute(
                                                      builder: (context) => ResultScreen()));
                                            }else if(listOfCountryandMrz[index] == 'Barcode'){
                                              Navigator.push(
                                                  context,
                                                  MaterialPageRoute(
                                                      builder: (context) => BarcodeList(barcode: barcodeList)));
                                            }
                                            else {
                                              mrzselected = 'other_mrz';
                                              print('other_mrz');
                                              await startMRZ();
                                              Navigator.push(
                                                  context,
                                                  MaterialPageRoute(
                                                      builder: (context) => ResultScreen()));
                                            }
                                          }
                                          else{
                                            var selected = sdkConfig!['countryList'].where((i)=>i['name'] == listOfCountryandMrz[index]).first;
                                            List<String> tempList = [];
                                            for(var item in selected['cards']){
                                              tempList.add(item['name']);
                                            }
                                            CardList.aOrientation = orientation;
                                            countrySelected = selected;
                                            Navigator.push(context, MaterialPageRoute(builder: (context)=>CardList(card: tempList, countrySelect: countrySelected)));
                                          }
                                        },
                                      )
                                  )
                              );
                          }),
                    ),
                  )
              ),
            ),
            )
        )
    );
  }
}