import Flutter
import UIKit
import AccuraOCR

struct gl {
    static var face1: UIImage? = nil
    static var face2: UIImage? = nil
    static var face1Detect: NSFaceRegion? = nil
    static var face2Detect: NSFaceRegion? = nil
    static var No = 0
}

public class SwiftOnlyForTrialPlugin: NSObject, FlutterPlugin {
    
    var goNativeCallBack: FlutterResult? = nil
    var accuraCameraWrapper: AccuraCameraWrapper? = nil
    var arrCountryList = NSMutableArray()
    var goNativeArgs: NSArray = []
    var accuraconfigs:NSDictionary = [:]
    var accuraTitleMsg:NSDictionary = [:]
    var viewController: FlutterViewController? = nil;
    
  
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "only_for_trial", binaryMessenger: registrar.messenger())
    let instance = SwiftOnlyForTrialPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }
    
    static func getDocumentsDirectory() -> URL {
        let paths = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask)
        return paths[0]
    }
    static func getImageFromUri(path: String) -> UIImage? {
        print(path)
        if let img = UIImage.init(contentsOfFile: path.replacingOccurrences(of: "file://", with: "")) {
            return img;
        }
        return nil
    }
    static func randomString(length: Int) -> String {
      let letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
      return String((0..<length).map{ _ in letters.randomElement()! })
    }
    static func getImageUri(img: UIImage, name: String?) -> String? {
        var file = randomString(length: 6)
        if let filename = name {
            file = filename
        }
        if let data = img.jpegData(compressionQuality: 1.0) {
            let filename = getDocumentsDirectory().appendingPathComponent("\(file).jpg")
            try? data.write(to: filename)
            print(filename.absoluteString)
            return filename.absoluteString
        }

        return nil
    }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      var args: NSArray = []
      if (call.arguments != nil) {
          let argsNew = call.arguments as! NSDictionary
          let jsonString = argsNew["arguments"] as? String
          let data: Data? = jsonString?.data(using: .utf8)
          args = ((try? JSONSerialization.jsonObject(with: data!, options: [])) as? NSArray)!
      }
      
      if (call.method == "getPlatformVersion") {
          result("iOS " + UIDevice.current.systemVersion)
      } else if (call.method == "getMetaData") {
          getMetaData(args, callback: result)
      }else if (call.method == "startOcrWithCard") {
          startOcrWithCard(args, callback: result)
      }else if(call.method == "startMRZ"){
          startMRZ(args,callback: result)
      }else if(call.method == "startFaceMatch"){
          startFaceMatch(args,callback: result)
      }else if(call.method == "startLiveness"){
          startLiveness(args,callback: result)
      }else if(call.method == "startBankCard"){
          startBankCard(args,callback: result)
      }else if(call.method == "startBarcode"){
          startBarcode(args,callback: result)
      }else if(call.method == "getGallery1") {
          openGallery(args, callback: result)
      }else if (call.method == "getGallery2") {
          openGallery2(args, callback: result)
      } else if (call.method == "getCamera1"){
          startCamera(args, callback: result)
      } else if (call.method == "getCamera2"){
          startCamera2(args, callback: result)
      }
      
  }

    
    func getMetaData(_ argsNew: NSArray, callback: @escaping FlutterResult){
        self.goNativeCallBack = callback
        self.goNativeArgs = argsNew;
        let configs = goNativeArgs[0] as? NSDictionary
        accuraTitleMsg = goNativeArgs[2] as! NSDictionary
        accuraconfigs = goNativeArgs[1] as! NSDictionary
        var results:[String: Any] = [:]
        results["isValid"] = false
        accuraCameraWrapper = AccuraCameraWrapper.init()
        accuraCameraWrapper?.setDefaultDialogs(true)
        accuraCameraWrapper?.showLogFile(true) // Set true to print log from KYC SDK
            let sdkModel = self.accuraCameraWrapper?.loadEngine(NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0] as String)
        if(sdkModel!.i >= 0){
                var countries:[Any] = []

                    results["isValid"] = true
                    if(sdkModel!.isMRZEnable)
                    {
                        results["isMRZEnable"] = true
                    }
                    else{
                        results["isMRZEnable"] = false
                    }
                    if(sdkModel!.isBankCardEnable) {
                        results["isBankCardEnable"] = true
                    } else {
                        results["isBankCardEnable"] = false
                    }
                    if(sdkModel!.isBarcodeEnable) {
                        results["isBarcodeEnable"] = true
                        
                        results["barcodes"] = ["ALL FORMATS","EAN-8","EAN-13","PDF417","AZTEC","CODE 128","CODE 93","CODE 39","DATA MATRIX","QR CODE","UPC-E","UPC-A","CODABAR"]//barcodes
                        
                    } else {
                        results["isBarcodeEnable"] = false
                    }
                let countryListStr = self.accuraCameraWrapper?.getOCRList()
                if(countryListStr != nil)
                {
                    for item in countryListStr!{
                        let cntry = item as! NSDictionary
                        var country:[String: Any] = [:]

                        country["name"] = cntry.value(forKey: "country_name")
                        country["id"] = cntry.value(forKey: "country_id")
                        var cards:[[String: Any]] = []
                        for cd in cntry.value(forKey: "cards") as! NSArray {
                            let cardF = cd as! NSDictionary
                            var card:[String: Any] = [:]
                            card["name"] = cardF.value(forKey:"card_name")
                            card["id"] = cardF.value(forKey:"card_id")
                            card["type"] = cardF.value(forKey:"card_type")
                            cards.append(card)
                        }
                        country["cards"] = cards
                        countries.append(country)
                    }
                    results["countryList"] = countries
                }
                else{
                }
            }else{
                results["isValid"] = false
                self.goNativeCallBack!(SwiftOnlyForTrialPlugin.convertJSONString(results:results))
            }
            if(sdkModel != nil){
            if sdkModel!.i > 0{
                if configs?["setFaceBlurPercentage"] != nil {
                    let val = (configs?["setFaceBlurPercentage"] as? Int32)!
                    self.accuraCameraWrapper?.setFaceBlurPercentage(val)
                }else{
                    self.accuraCameraWrapper?.setFaceBlurPercentage(80)
                }
                if configs?["setHologramDetection"] != nil {
                    let val = (configs?["setHologramDetection"] as? Bool)!
                    self.accuraCameraWrapper?.setHologramDetection(val)
                }else{
                    self.accuraCameraWrapper?.setHologramDetection(true)
                }
                if configs?["setLowLightTolerance"] != nil {
                    let val = (configs?["setLowLightTolerance"] as? Int32)!
                    self.accuraCameraWrapper?.setLowLightTolerance(val)
                }else{
                    self.accuraCameraWrapper?.setLowLightTolerance(10)
                }
                if configs?["setMotionThreshold"] != nil {
                    let val = (configs?["setMotionThreshold"] as? Int32)!
                    self.accuraCameraWrapper?.setMotionThreshold(val)
                }else{
                    self.accuraCameraWrapper?.setMotionThreshold(25)
                }
                if configs?["setMinGlarePercentage"] != nil && configs?["setMaxGlarePercentage"] != nil {
                    let val1 = (configs?["setMinGlarePercentage"] as? Int32)!
                    let val2 = (configs?["setMaxGlarePercentage"] as? Int32)!

                    self.accuraCameraWrapper?.setGlarePercentage(val1, intMax: val2)
                }else{
                    self.accuraCameraWrapper?.setGlarePercentage(6, intMax: 99)
                }
                if configs?["setBlurPercentage"] != nil {
                    let val = (configs?["setBlurPercentage"] as? Int32)!
                    self.accuraCameraWrapper?.setBlurPercentage(val)
                }else{
                    self.accuraCameraWrapper?.setBlurPercentage(60)
                }
                if configs?["setCameraFacing"] != nil {
                    if configs?["setCameraFacing"] as? Int == 0{
                        self.accuraCameraWrapper?.setCameraFacing(.CAMERA_FACING_BACK)
                    }else if configs?["setCameraFacing"] as? Int == 1{
                        self.accuraCameraWrapper?.setCameraFacing(.CAMERA_FACING_FRONT)
                    }
                }else{
                    self.accuraCameraWrapper?.setCameraFacing(.CAMERA_FACING_BACK)
                }
                accuraCameraWrapper?.setCheckPhotoCopy(((configs!["isCheckPhotoCopy"] as? Bool)!))
            }
        }
        self.goNativeCallBack!(SwiftOnlyForTrialPlugin.convertJSONString(results:results))
    }
    
    func startFaceMatch(_ argsNew: NSArray, callback: @escaping FlutterResult){
        self.goNativeCallBack = callback
        self.goNativeArgs = argsNew;
        let face:NSDictionary
        face = goNativeArgs[0] as! NSDictionary
        let fmInit = EngineWrapper.isEngineInit()
        if !fmInit{
            EngineWrapper.faceEngineInit()
        }
        let fmValue = EngineWrapper.getEngineInitValue() //get engineWrapper load status
        if fmValue == -20{
            goNativeCallBack!(FlutterError.init(code: "101", message: "key not found", details: nil))
            sendError(msg: "key not found")
        }else if fmValue == -15{
            goNativeCallBack!(FlutterError.init(code: "101", message: "License Invalid", details: nil))

            sendError(msg: "License Invalid")
        }else{
            let FMController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "LVController") as! LVController
            FMController.faceArgs = goNativeArgs[1] as! NSDictionary
            FMController.isfacematch = true
            FMController.callBack = self.goNativeCallBack
            FMController.faceImage = SwiftOnlyForTrialPlugin.getImageFromUri(path: face["face_uri"] as! String)
            getTopMostViewController()?.present(FMController, animated: true, completion: nil)
        }
    }
    
    func startLiveness(_ argsNew: NSArray, callback: @escaping FlutterResult){
        self.goNativeCallBack = callback
        self.goNativeArgs = argsNew;
        let face:NSDictionary
        face = goNativeArgs[0] as! NSDictionary
        let fmInit = EngineWrapper.isEngineInit()
        if !fmInit{
            EngineWrapper.faceEngineInit()
        }
        let fmValue = EngineWrapper.getEngineInitValue() //get engineWrapper load status
        if fmValue == -20{
            goNativeCallBack!(FlutterError.init(code: "101", message: "key not found", details: nil))
            sendError(msg: "key not found")
        }else if fmValue == -15{
            goNativeCallBack!(FlutterError.init(code: "101", message: "License Invalid", details: nil))
            sendError(msg: "License Invalid")
        }else{
            let LVController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "LVController") as! LVController
            LVController.isLiveness = true
            LVController.livenessConfig = goNativeArgs[1] as! NSDictionary
            LVController.callBack = self.goNativeCallBack
            LVController.faceImage = SwiftOnlyForTrialPlugin.getImageFromUri(path: face["face_uri"] as! String)
            getTopMostViewController()?.present(LVController, animated: true, completion: nil)
        }
    }
    
    func startOcrWithCard(_ argsNew: NSArray, callback: @escaping FlutterResult){
        self.goNativeCallBack = callback;
        self.goNativeArgs = argsNew;
        let viewController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "ViewController") as! ViewController
        viewController.countryid = goNativeArgs[1] as? Int
        viewController.cardid = goNativeArgs[2] as? Int
        viewController.isCheckScanOCR = true
        if self.goNativeArgs[4] as! Int == 1 {
            viewController.isBarCode = true
            viewController.isBarcodeEnabled = false
        }
        viewController.cardType = self.goNativeArgs[4] as? Int
        viewController.docName = self.goNativeArgs[3] as! String
        viewController.callBack = self.goNativeCallBack
        viewController.accuraTitleMsg = accuraTitleMsg
        viewController.accuraErrorCode = accuraconfigs
        viewController.modalPresentationStyle = .fullScreen
        viewController.modalTransitionStyle = .coverVertical
        getTopMostViewController()?.present(viewController, animated: true, completion: nil)
    }
    
    func startBankCard(_ argsNew: NSArray,callback: @escaping FlutterResult){
        self.goNativeCallBack = callback
        let viewController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "ViewController") as! ViewController
        viewController.isCheckScanOCR = true
        viewController.cardType = 3
        viewController.accuraTitleMsg = accuraTitleMsg
        viewController.accuraErrorCode = accuraconfigs
        viewController.callBack = self.goNativeCallBack
        viewController.modalPresentationStyle = .fullScreen
        viewController.modalTransitionStyle = .coverVertical
        getTopMostViewController()?.present(viewController, animated: true, completion: nil)
    }
    
    func startBarcode(_ argsNew: NSArray, callback: @escaping FlutterResult) {

        self.goNativeCallBack = callback;
        self.goNativeArgs = argsNew;
        
            let viewController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "ViewController") as! ViewController
        viewController.accuraTitleMsg = accuraTitleMsg
        viewController.accuraErrorCode = accuraconfigs
            viewController.callBack = self.goNativeCallBack
            viewController.isBarCode = true
        viewController.isBarcodeEnabled = true
            viewController.reactViewController = self.viewController
        viewController.selectedTypes = setSelectedTypes(types: goNativeArgs[0] as! String)
            viewController.modalPresentationStyle = .fullScreen
            viewController.modalTransitionStyle = .coverVertical
            getTopMostViewController()?.present(viewController, animated: true, completion: nil)

    }
    
    func startMRZ(_ argsNew: NSArray, callback: @escaping FlutterResult){
        self.goNativeCallBack = callback;
        self.goNativeArgs = argsNew;
        let viewController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "ViewController") as! ViewController
        if goNativeArgs[1] as! String == "passport" {
            viewController.MRZDocType = 1
        }else if goNativeArgs[1] as! String == "visa_card" {
            viewController.MRZDocType = 3
        }else if goNativeArgs[1] as! String == "id_mrz" {
            viewController.MRZDocType = 2
        }else{
            viewController.MRZDocType = 0
        }
        viewController.callBack = self.goNativeCallBack
        viewController.isCheckScanOCR = false
        viewController.reactViewController = self.viewController
        viewController.accuraTitleMsg = accuraTitleMsg
        viewController.accuraErrorCode = accuraconfigs
        viewController.modalPresentationStyle = .fullScreen
        viewController.modalTransitionStyle = .coverVertical
        DispatchQueue.main.async {
            self.getTopMostViewController()?.present(viewController, animated: true, completion: nil)
        }
}
    
    func startCamera2(_ argsNew: NSArray,callback: @escaping FlutterResult){
        self.goNativeCallBack = callback;
        self.goNativeArgs = argsNew;
        let fmInit = EngineWrapper.isEngineInit()
        if !fmInit{
            EngineWrapper.faceEngineInit()
        }
        let fmValue = EngineWrapper.getEngineInitValue() //get engineWrapper load status
        if fmValue == -20{
            sendError(msg: "Key not found")
            return
        }else if fmValue == -15{
            sendError(msg: "License Invalid")
            return
        }
        let accuraConfigs = self.goNativeArgs[0] as! [String:Any]
        let face1 = SwiftOnlyForTrialPlugin.getImageFromUri(path:accuraConfigs["face1"] as! String)
        let face2 = SwiftOnlyForTrialPlugin.getImageFromUri(path:accuraConfigs["face2"] as! String)
        gl.face1 = face1
        gl.face2 = face2
        gl.No = 2
        DispatchQueue.main.async {
            let FMController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "FMController") as! FMController
            FMController.callBack = self.goNativeCallBack
            FMController.facematch2 = true
            FMController.faceArgs = self.goNativeArgs[1] as! NSDictionary

            FMController.reactViewController = self.viewController

            DispatchQueue.main.async {
                let nav = UINavigationController(rootViewController: FMController)
                nav.isNavigationBarHidden = true
                nav.modalPresentationStyle = .fullScreen
                nav.modalTransitionStyle = .coverVertical
                self.getTopMostViewController()?.present(nav, animated: true, completion: {
                    
                })
            }
        }
    }
    
    func startCamera(_ argsNew: NSArray,callback: @escaping FlutterResult){
        self.goNativeCallBack = callback;
        self.goNativeArgs = argsNew;
        let fmInit = EngineWrapper.isEngineInit()
        if !fmInit{
            EngineWrapper.faceEngineInit()
        }
        let fmValue = EngineWrapper.getEngineInitValue() //get engineWrapper load status
        if fmValue == -20{
            sendError(msg: "Key not found")
            return
        }else if fmValue == -15{
            sendError(msg: "License Invalid")
            return
        }
        let accuraConfigs = self.goNativeArgs[0] as! [String:Any]
        let face1 = SwiftOnlyForTrialPlugin.getImageFromUri(path: accuraConfigs["face1"] as! String)
        let face2 = SwiftOnlyForTrialPlugin.getImageFromUri(path: accuraConfigs["face2"] as! String)
        gl.face1 = face1
        gl.face2 = face2
        gl.No = 1
        DispatchQueue.main.async {
            let FMController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "FMController") as! FMController
            FMController.callBack = self.goNativeCallBack
            FMController.faceMatch1 = true
            FMController.faceArgs = self.goNativeArgs[1] as! NSDictionary

            FMController.reactViewController = self.viewController

            DispatchQueue.main.async {
                let nav = UINavigationController(rootViewController: FMController)
                nav.isNavigationBarHidden = true
                nav.modalPresentationStyle = .fullScreen
                nav.modalTransitionStyle = .coverVertical
                self.getTopMostViewController()?.present(nav, animated: true, completion: {
                    
                })
            }
        }
    }
    
    func openGallery(_ argsNew: NSArray, callback: @escaping FlutterResult){
        self.goNativeCallBack = callback;
        self.goNativeArgs = argsNew;
        let fmInit = EngineWrapper.isEngineInit()
        if !fmInit{
            EngineWrapper.faceEngineInit()
        }
        let fmValue = EngineWrapper.getEngineInitValue() //get engineWrapper load status
        if fmValue == -20{
            goNativeCallBack!(FlutterError.init(code: "101", message: "Key not found", details: nil))
            sendError(msg: "Key not found")
            return
        }else if fmValue == -15{
            goNativeCallBack!(FlutterError.init(code: "101", message: "License Invalid", details: nil))
            sendError(msg: "License Invalid")
            return
        }
        let accuraConfigs = self.goNativeArgs[0] as! [String:Any]
        let face1 = SwiftOnlyForTrialPlugin.getImageFromUri(path:accuraConfigs["face1"] as! String)
        let face2 = SwiftOnlyForTrialPlugin.getImageFromUri(path: accuraConfigs["face2"] as! String)
        gl.face1 = face1
        gl.face2 = face2
        gl.No = 1
        DispatchQueue.main.async {
            let FMController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "FMController") as! FMController
            FMController.callBack = self.goNativeCallBack
            FMController.gallery1 = true
            FMController.reactViewController = self.viewController
//            FMController.win = self.viewControllerWindow

            DispatchQueue.main.async {
                let nav = UINavigationController(rootViewController: FMController)
                nav.isNavigationBarHidden = true
                nav.modalPresentationStyle = .fullScreen
                nav.modalTransitionStyle = .coverVertical
                self.getTopMostViewController()?.present(nav, animated: true, completion: {
                    
                })
            }
        }
    }
    func openGallery2(_ argsNew: NSArray, callback: @escaping FlutterResult){
        self.goNativeCallBack = callback;
        self.goNativeArgs = argsNew;
        let fmInit = EngineWrapper.isEngineInit()
        if !fmInit{
            EngineWrapper.faceEngineInit()
        }
        let fmValue = EngineWrapper.getEngineInitValue() //get engineWrapper load status
        if fmValue == -20{
            goNativeCallBack!(FlutterError.init(code: "101", message: "Key not found", details: nil))
            sendError(msg: "Key not found")
            return
        }else if fmValue == -15{
            goNativeCallBack!(FlutterError.init(code: "101", message: "License Invalid", details: nil))
            sendError(msg: "License Invalid")
            return
        }
        let accuraConfigs = self.goNativeArgs[0] as! [String:Any]
        let face1 = SwiftOnlyForTrialPlugin.getImageFromUri(path: accuraConfigs["face1"] as! String)
        let face2 = SwiftOnlyForTrialPlugin.getImageFromUri(path: accuraConfigs["face2"] as! String)
        gl.face1 = face1
        gl.face2 = face2
        gl.No = 2
        DispatchQueue.main.async {
            let FMController = UIStoryboard(name: "MainStoryboard_iPhone", bundle: nil).instantiateViewController(withIdentifier: "FMController") as! FMController
            FMController.callBack = self.goNativeCallBack
            FMController.gallery2 = true
            FMController.reactViewController = self.viewController
//            FMController.win = self.viewControllerWindow

            DispatchQueue.main.async {
                let nav = UINavigationController(rootViewController: FMController)
                nav.isNavigationBarHidden = true
                nav.modalPresentationStyle = .fullScreen
                nav.modalTransitionStyle = .coverVertical
                self.getTopMostViewController()?.present(nav, animated: true, completion: {
                    
                })
            }
        }
    }

    
    static func convertJSONString(results: [String: Any]) -> String {
        
        if let theJSONData = try?  JSONSerialization.data( withJSONObject: results, options: .prettyPrinted ),
          let theJSONText = String(data: theJSONData, encoding: String.Encoding.ascii) {
              print("JSON string = \n\(theJSONText)")
            return theJSONText.components(separatedBy: .whitespacesAndNewlines).joined();
        }
        return "{}"
    }
    
    func sendError(msg: String) {
        
        self.goNativeCallBack!([msg as Any, NSNull()])
    }
    
    func getTopMostViewController() -> UIViewController? {
        var topMostViewController = UIApplication.shared.keyWindow?.rootViewController

        while let presentedViewController = topMostViewController?.presentedViewController {
            topMostViewController = presentedViewController
        }

        return topMostViewController
    }
    
    func setSelectedTypes(types: String) -> BarcodeType {
        switch types
        {
        case "ALL FORMATS":
            return .all
        case "EAN-8":
            return .ean8
        case "EAN-13":
            return .ean13
        case "PDF417":
            return .pdf417
        case "AZTEC":
            return .aztec
        case "CODE 128":
            return .code128
        case "CODE 39":
            return .code39
        case "CODE 93":
            return .code93
        case "DATA MATRIX":
            return .dataMatrix
        case "ITF":
            return .itf
        case "QR CODE":
            return .qrcode
        case "UPC-E":
            return .upce
        case "UPC-A":
            return .upca
        case "CODABAR":
            return .codabar
        default:
            return .all
        }
    }
}

